#**********************************************************************************************************+++++++++++++++*********************!
# (c) nextnano GmbH                                                                                                                            !
#                                                                                                                                              !
# This is ascript for the following tutorial:                                                                                                  !
#   at https://www.nextnano.com/documentation/tools/nextnanomat/export_functionalities/paraview/python_2d.html#nnmt-export-paraview-python-2d  !
#                                                                                                                                              !
# For comments or feedback, please contact support@nextnano.com                                                                                !
#**********************************************************************************************************************************************!

import subprocess
import sys

from os.path import normpath, dirname, join
from nn_ParaView_GUI import nn_PrV_GUI



#----------------- Constants and default settings --------------------------------------------

## Configuration parameters in the script "nn_ParaView_Plotter.py" (it must not be changed)
RESCALE_OPTION = False 
OPACITY_VALUE = 0.22
MAXIMUM_NUMBER_DATASETS_VALUE = 8

#Setting list in the GUI ( do not change the keys )
settingList = {
    'RESCALE_OPTION' : 'False',
    'WARP' : 'False',
    'OVERLAY': 'False', 
    'SCREENSHOT' : 'False',
    'SOLID_COLOR' : 'False',
    'OPACITY': 'False',
    'CAMERA_VIEW_UP_Y': 'False',
    'MAXIMUM_NUMBER_DATASETS': 'False',
    'LINK_WARPED_PLOTS' : 'False',
    'OTHER' : 'False'
    }
    

#------------------------------- PATH OF OTHER SCRIPTS and THE SELECTED PLOT -------------------                  

# Paths
#Folder where the Plotter and GUI scripts are stored (it shall be in the same directory)
scriptDirectory = dirname(sys.argv[1])

#Script for plotting and modified copy ( output )
originalScript = "nn_ParaView_Plotter.py"
templateFile = sys.argv[2][:-1] + originalScript
scriptFile = sys.argv[2][:-1] + originalScript.replace(".py","_output.py")

print('\nScript to be processed in ParaView')
print(scriptFile)

#Path of the files to be plotted and overlay
path = sys.argv[4]


#------------------------------- DISPLAY GUI --------------------------------------------------

#Call the GUI for selecting files to be plotted and overlay file, and modify options
#path will be preselected in the interface and used to find its simulation Folder
root = nn_PrV_GUI(path) 
root.mainloop()


#------------------------------- PROCESSING RESULTS FROM GUI ----------------------------------

#Return of the selections captured by the GUI
selectedFiles = root.getSelectedFiles()
optionList = root.getSettingOptions()
simulationFolder = root.getSimulationFolder()

scriptFile = join(simulationFolder,originalScript.replace(".py","_output.py"))

print('\nScript to be processed in Paraview')
print(scriptFile) 
# Update settingList with the new values captured from the GUI
for key in settingList.keys():
    if key in optionList.keys():
        settingList[key] = optionList[key]

maximumNumberDatasets = root.getNumberDatasets()
opacity = root.getOpacity()
# to DO: it still will require to validate these fields in the GUI


if settingList['OVERLAY'] == "True":
    overlayFile = root.selectedOverlayFile

# Generate the strings to be modified in the original (template) script
fileListString = "[\n" 

if len(selectedFiles) > 0:
    for file in selectedFiles:
        fileListString = fileListString + '        normpath(\'' + file + '\'),\n    '
else:
    fileListString = fileListString + "\"\""

fileListString = fileListString + "]\n\n"


if settingList['OVERLAY'] == "True":     
    overlayFileString = "normpath(\'" + overlayFile +  "\')\n\n"
else:
    overlayFileString = "\"\"\n"


if settingList['SCREENSHOT'] == "True":     
    imageDirString = "normpath(\'" + join(simulationFolder, 'PrV Images') +  "\')\n\n"
else:
    imageDirString = "\"\"\n"



#------------------------------- GENERATE PLOTTER SCRIPT FROM TEMPLATE ------------------------

# Read original script
with open(normpath(templateFile),"r") as inputScript:
    string = inputScript.read()
   
# Modify and save new script using the updated settingsList
with open(normpath(scriptFile), "w") as outputScript:
 #   string = string.replace('RESCALE_OPTION = False',  'RESCALE_OPTION = ' + str(RESCALE_OPTION))

    for key in settingList.keys():
        string = string.replace(key + " = False", key + ' = ' + settingList[key])

    if settingList['MAXIMUM_NUMBER_DATASETS'] == 'True' and maximumNumberDatasets != 8:
        string = string.replace('MAXIMUM_NUMBER_DATASETS = False', 'MAXIMUM_NUMBER_DATASETS_VALUE = ' + settingList['MAXIMUM_NUMBER_DATASETS'])
        string = string.replace('MAXIMUM_NUMBER_DATASETS_VALUE = 8', 'MAXIMUM_NUMBER_DATASETS_VALUE = ' + str(maximumNumberDatasets)) 
        
    if settingList['OPACITY'] == 'True' and opacity != OPACITY_VALUE :
        string = string.replace('OPACITY = False', 'OPACITY = ' + settingList['OPACITY'])  
        string = string.replace('OPACITY_VALUE = ' + str(OPACITY_VALUE), 'OPACITY_VALUE = ' + str(opacity) )
        
    string = string.replace('IMPORTED_FILE_LIST', fileListString)
    string = string.replace('IMPORTED_OVERLAY_FILE', overlayFileString)
    string = string.replace('IMPORTED_IMAGE_DIRECTORY', imageDirString)

    outputScript.write(string)


#------------------------------- LAUNCH ParaView  ---------------------------------------------

executable = ''

if len(sys.argv) > 7:
    executable = sys.argv[8]

if executable == '':
    executable = 'paraview'

process = subprocess.Popen([executable,"--script=" + scriptFile])
